"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client = require("./client");
class BaseAction {
    constructor() {
        this.beforeChangeHandlers = [];
        this.afterChangeHandlers = [];
    }
    onChanged(handler) {
        this.afterChangeHandlers.push(handler);
    }
    onBeforeChange(handler) {
        this.beforeChangeHandlers.push(handler);
    }
    async fireOnBeforeChange(issue) {
        for (const before of this.beforeChangeHandlers) {
            await before(issue);
        }
    }
    async fireOnChanged(issue) {
        for (const after of this.afterChangeHandlers) {
            await after(issue);
        }
    }
}
exports.BaseAction = BaseAction;
class BaseIssueAction extends BaseAction {
    constructor(issue) {
        super();
        this.issue = issue;
    }
    async fireOnBeforeChange() {
        await super.fireOnBeforeChange(this.issue);
    }
    async fireOnChanged() {
        await super.fireOnChanged(this.issue);
    }
}
exports.BaseIssueAction = BaseIssueAction;
var PullRequests;
(function (PullRequests) {
    class Base extends BaseAction {
        constructor(pr) {
            super();
            this.pr = pr;
        }
    }
    PullRequests.Base = Base;
    class Merge extends Base {
        get summary() {
            return `Merge pull request ${this.pr.fullName}`;
        }
        async execute(_info) {
            await client.mergePR(this.pr, await this.getMergeOptions());
        }
        async getMergeOptions() {
            const { pr } = this;
            const canMerge = await pr.getMergeableState();
            if (!canMerge) {
                throw new Error("TODO");
            }
            const commits = await pr.getCommitsRaw();
            const title = `Merge PR #${pr.number}: ${pr.title}`;
            let message = "";
            for (const commit of commits) {
                message += `* ${commit.commit.message}\n`;
            }
            const sha = commits[commits.length - 1].sha;
            return { title, message, sha };
        }
    }
    PullRequests.Merge = Merge;
})(PullRequests = exports.PullRequests || (exports.PullRequests = {}));
var Labels;
(function (Labels) {
    class Base extends BaseIssueAction {
        constructor(issue, labels) {
            super(issue);
            this.labels = labels;
        }
    }
    Labels.Base = Base;
    class Add extends Base {
        get summary() {
            return `Add labels ${JSON.stringify(this.labels)} to issue ${this.issue.number}`;
        }
        async execute(_info) {
            const labelsToAdd = this.labels.filter((lab) => !this.issue.hasLabel(lab));
            if (labelsToAdd.length === 0)
                return;
            await this.fireOnBeforeChange();
            await client.addLabels(this.issue, labelsToAdd);
            await this.fireOnChanged();
        }
    }
    Labels.Add = Add;
    class Remove extends Base {
        get summary() {
            return `Remove labels ${JSON.stringify(this.labels)} to issue ${this.issue.number}`;
        }
        async execute(_info) {
            const labelsToRemove = this.labels.filter((lab) => this.issue.hasLabel(lab));
            if (labelsToRemove.length === 0)
                return;
            await this.fireOnBeforeChange();
            await client.removeLabels(this.issue, this.labels);
            await this.fireOnChanged();
        }
    }
    Labels.Remove = Remove;
    class Set extends Base {
        get summary() {
            return `Apply label set ${JSON.stringify(this.labels)} to issue ${this.issue.number}`;
        }
        async execute(_info) {
            const desired = this.labels.slice().sort();
            const actual = this.issue.labels.map((l) => l.name).sort();
            if (JSON.stringify(desired) === JSON.stringify(actual)) {
                return;
            }
            await this.fireOnBeforeChange();
            await client.setLabels(this.issue, this.labels);
            await this.fireOnChanged();
        }
    }
    Labels.Set = Set;
})(Labels = exports.Labels || (exports.Labels = {}));
var Comments;
(function (Comments) {
    function parseHeader(body) {
        const regex = /^<!--header (.*) headerend-->/g;
        const match = regex.exec(body);
        return match ? JSON.parse(match[1]) : undefined;
    }
    function makeHeader(header) {
        return `<!--header ${JSON.stringify(header)} headerend-->`;
    }
    function makeFooter() {
        return `<!--footer bot @ ${(new Date()).toLocaleString()} footerend-->`;
    }
    function makeComment(slug, body) {
        return makeHeader({ slug }) + "\r\n" + body + "\r\n" + makeFooter();
    }
    function getBody(comment) {
        const regex = /headerend-->\r?\n(.*)\r?\n<!--footer/;
        const match = regex.exec(comment.body);
        if (match) {
            return match[1];
        }
        return comment.body;
    }
    class Base extends BaseIssueAction {
        constructor(issue, slug) {
            super(issue);
            this.slug = slug;
        }
    }
    Comments.Base = Base;
    class Add extends Base {
        constructor(issue, slug, body) {
            super(issue, slug);
            this.body = body;
        }
        get summary() {
            return `Write comment (slug '${this.slug}') on issue ${this.issue.number}`;
        }
        async execute(_info) {
            const me = await client.getMyLogin();
            // Find my comments, if it exists
            const comments = (await this.issue.getComments()).filter((c) => c.user.login === me);
            for (const c of comments) {
                const header = parseHeader(c.body);
                if (header && (header.slug === this.slug)) {
                    const body = getBody(c);
                    if (body !== this.body) {
                        console.log("Actual: " + body);
                        console.log("Desired: " + this.body);
                        await this.fireOnBeforeChange();
                        await client.editComment(c, makeComment(this.slug, this.body));
                        await this.fireOnChanged();
                    }
                    return;
                }
            }
            await this.fireOnBeforeChange();
            await client.addComment(this.issue, makeComment(this.slug, this.body));
            await this.fireOnChanged();
        }
    }
    Comments.Add = Add;
})(Comments = exports.Comments || (exports.Comments = {}));
var Assignees;
(function (Assignees) {
    class Base extends BaseIssueAction {
        constructor(issue, assignees) {
            super(issue);
            this.assignees = assignees;
        }
    }
    Assignees.Base = Base;
    class Add extends Base {
        get summary() {
            return `Assign issue ${this.issue.fullName} to ${JSON.stringify(this.assignees)}`;
        }
        async execute(_info) {
            throw new Error("Not implemented");
        }
    }
    Assignees.Add = Add;
})(Assignees = exports.Assignees || (exports.Assignees = {}));
var Issues;
(function (Issues) {
    class SetColumn extends BaseIssueAction {
        constructor(issue, project, column) {
            super(issue);
            this.project = project;
            this.column = column;
        }
        get summary() {
            const column = this.column ? this.column.name : "(none)";
            return `Move issue ${this.issue.fullName} to column ${column} in project ${this.project.projectId}`;
        }
        async execute(_info) {
            await this.project.doSetIssueColumn(this.issue, this.column);
        }
    }
    Issues.SetColumn = SetColumn;
    class Lock extends BaseIssueAction {
        get summary() {
            return `Lock issue ${this.issue.fullName}`;
        }
        async execute(_info) {
            if (this.issue.locked)
                return;
            await this.fireOnBeforeChange();
            await client.lockIssue(this.issue);
            await this.fireOnChanged();
        }
    }
    Issues.Lock = Lock;
    class Unlock extends BaseIssueAction {
        get summary() {
            return `Unlock issue ${this.issue.fullName}`;
        }
        async execute(_info) {
            if (!this.issue.locked)
                return;
            await this.fireOnBeforeChange();
            await client.unlockIssue(this.issue);
            await this.fireOnChanged();
        }
    }
    Issues.Unlock = Unlock;
    class Close extends BaseIssueAction {
        get summary() {
            return `Close issue ${this.issue.fullName}`;
        }
        async execute(_info) {
            if (this.issue.state === "open") {
                await this.fireOnBeforeChange();
                await client.closeIssue(this.issue);
                await this.fireOnChanged();
            }
        }
    }
    Issues.Close = Close;
    class Reopen extends BaseIssueAction {
        get summary() {
            return `Reopen issue ${this.issue.fullName}`;
        }
        async execute(_info) {
            if (this.issue.state === "closed") {
                await this.fireOnBeforeChange();
                await client.reopenIssue(this.issue);
                await this.fireOnChanged();
            }
        }
    }
    Issues.Reopen = Reopen;
})(Issues = exports.Issues || (exports.Issues = {}));
//# sourceMappingURL=action.js.map