import moment = require("moment");
import { IAction } from "./action";
import { Cache } from "./cache";
import * as api from "./github-api";
export declare function useCache(c: Cache): void;
export declare function parseBasicRepoReference(ownerSlashName: string): Repository;
export declare function parseRepoReferenceFromURL(url: string): Repository;
export declare class User {
    private static readonly pool;
    static fromLogin(login: string): Promise<User>;
    static fromData(data: api.User): User;
    readonly login: string;
    static getCacheKey(login: string): string;
    getCacheKey(): string;
    private constructor();
    update(data: api.User): void;
}
export declare class Label {
    readonly repo: api.RepoReference;
    private static pool;
    static fromData(repo: api.RepoReference, data: api.Label): Label;
    readonly name: string;
    readonly color: string;
    private constructor();
    update(data: [api.RepoReference, api.Label]): void;
}
export declare class IssueComment {
    private originalData;
    readonly issue: Issue;
    readonly id: number;
    readonly user: User;
    readonly created_at: moment.Moment;
    readonly updated_at: moment.Moment;
    readonly body: string;
    readonly repository: Repository;
    static fromData(originalData: api.IssueComment, issue: Issue): IssueComment;
    private constructor();
    update(data: api.IssueComment): void;
    getReactions(): Promise<api.Reaction[]>;
}
export declare class Milestone {
    /** Internal milestone id */
    readonly id: number;
    /** This is the actual user-facing number */
    readonly number: number;
    /** Title  */
    readonly title: string;
    /** Description  */
    readonly description: string;
    /** State can be "open" or "closed" */
    readonly state: "open" | "closed";
    /** Who created this milestone */
    readonly creator: User;
    /** When this milestone was created */
    readonly created_at: moment.Moment;
    /** When this milestone was last updated */
    readonly updated_at: moment.Moment;
    /** When this milestone is due, if applicable */
    readonly due_on: moment.Moment | null;
    /** When this milestone was closed, if applicable */
    readonly closed_at: moment.Moment | null;
    constructor(originalData: api.Milestone);
}
export declare class Repository {
    readonly owner: string;
    readonly name: string;
    private static cache;
    static create(owner: string, name: string): Repository;
    readonly reference: api.RepoReference;
    private constructor();
}
export declare abstract class IssueOrPullRequest {
    static fromData(data: api.Issue): Promise<Issue | PullRequest>;
    static getCacheKey(repo: api.RepoReference, number: number, isPR: boolean): string;
    static getCommentsCacheKey(repo: api.RepoReference, number: number, isPR: boolean): string;
    static getCacheKeyBasePath(repo: api.RepoReference, number: number, isPR: boolean): string;
    /** This is the actual user-facing number */
    readonly number: number;
    /** An internal GitHub ID number */
    readonly id: number;
    /** Title */
    readonly title: string;
    /** The main body of the issue */
    readonly body: string;
    /** State can be "open" or "closed" */
    readonly state: "open" | "closed";
    /** Whether the issue has been locked or not */
    readonly locked: boolean;
    readonly html_url: string;
    /** When this issue was created */
    readonly created_at: moment.Moment;
    /** When this issue was last updated */
    readonly updated_at: moment.Moment;
    /** When this issue was closed, if it's closed */
    readonly closed_at: moment.Moment | null;
    /** The labels this issue has */
    readonly labels: ReadonlyArray<Label>;
    /** The author of this issue */
    readonly user: User;
    /** A list, possibly empty, of who is assigned to this issue */
    readonly assignees: ReadonlyArray<User>;
    /** A milestone, if one is assigned */
    readonly milestone: Milestone | null;
    readonly abstract isPullRequest: boolean;
    readonly repository: Repository;
    protected constructor(originalData: api.Issue);
    /** Returns a string like 'Microsoft/TypeScript#14' */
    readonly fullName: string;
    update(): void;
    getComments(): Promise<IssueComment[]>;
    /**
     * Adds a label to an issue or PR.
     */
    addLabel(...labels: string[]): IAction;
    /**
     * Adds labels to an issue or PR.
     */
    addLabels(labels: string[]): IAction;
    /**
     * Deletes a label from an issue.
     */
    removeLabel(...labels: string[]): IAction;
    /**
     * Deletes labels from an issue.
     */
    removeLabels(labels: string[]): IAction;
    /**
     * Sets, exactly, which labels are on an issue
     */
    setLabels(...labels: string[]): IAction;
    /**
     * Shortcut method for setting/clearing labels with an object literal, e.g.
     * issue.setHasLabels({
     *  "Needs Info": false,
     *  "Ready for Triage": true
     * }).
     * You can specify a null value to cause nothing to happen either way
     */
    setHasLabels(labelMap: {
        [key: string]: boolean | null;
    }): void;
    /**
     * Returns true if this issue has the specified label
     */
    hasLabel(labelName: string | Label): boolean;
    /**
     * Adds or updates a comment with the specified slug to the issue
     */
    addComment(slug: string, body: string): IAction;
    lock(): IAction;
    unlock(): IAction;
    close(): IAction;
    reopen(): IAction;
}
export declare class Issue extends IssueOrPullRequest {
    static fromData(_data: api.Issue): never;
    static fromIssueData(data: api.Issue): Issue;
    readonly isPullRequest: boolean;
    private constructor();
}
export declare type StatusSummary = "pass" | "fail" | "pending";
export declare class PullRequest extends IssueOrPullRequest {
    static fromReference(repo: api.RepoReference, number: number | string): Promise<PullRequest>;
    static fromData(_data: api.Issue): never;
    static getPRCacheKey(repo: api.RepoReference, number: number, isPR: boolean): string;
    static fromIssueAndPRData(prData: api.PullRequest, issueData: api.Issue): Promise<PullRequest>;
    static fromPullRequestData(prData: api.PullRequest, issueData: api.Issue): PullRequest;
    readonly head: api.Commit;
    /**
     * See comment at https://developer.github.com/v3/pulls/#get-a-single-pull-request
     */
    readonly merge_commit_sha: string;
    readonly merged: boolean;
    mergeable: boolean | null;
    readonly mergeable_state: api.MergeableState;
    readonly comments: number;
    readonly commits: number;
    readonly additions: number;
    readonly deletions: number;
    readonly changed_files: number;
    readonly isPullRequest: boolean;
    private constructor();
    getStatusSummary(): Promise<api.StatusSummary>;
    getStatus(): Promise<api.CombinedStatus>;
    getReviews(): Promise<api.PullRequestReview[]>;
    getCommitsRaw(): Promise<api.PullRequestCommit[]>;
    getFilesRaw(): Promise<api.PullRequestFile[]>;
    getMergeableState(): Promise<boolean | null>;
    merge(): IAction;
}
export declare class Project {
    readonly projectId: number;
    readonly columns: ProjectColumn[];
    static create(projectId: number): Promise<Project>;
    private constructor();
    setIssueColumn(issue: IssueOrPullRequest, targetColumn: ProjectColumn | undefined): IAction;
    doSetIssueColumn(issue: IssueOrPullRequest, targetColumn: ProjectColumn | undefined): Promise<void>;
    findColumnByName(name: string | RegExp): ProjectColumn | undefined;
}
export declare class ProjectColumn {
    readonly columnId: number;
    readonly name: string;
    static create(columnId: number, name: string): Promise<ProjectColumn>;
    cards: ProjectCard[];
    private constructor();
    findProjectCardForIssue(issue: IssueOrPullRequest): ProjectCard | undefined;
}
export declare class ProjectCard {
    private readonly data;
    readonly id: number;
    constructor(data: api.ProjectColumnCard);
    getIssueNumber(): number | undefined;
}
