"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require("moment");
const sleep = require("sleep-promise");
const action_1 = require("./action");
const actionRunner_1 = require("./actionRunner");
const build_path_1 = require("./build-path");
const client = require("./client");
const weak_string_map_1 = require("./weak-string-map");
function useCache(c) {
    cache = c;
}
exports.useCache = useCache;
let cache;
function parseDate(s) {
    if (s === null)
        return null;
    return moment(new Date(s));
}
function parseBasicRepoReference(ownerSlashName) {
    const m = /^([\w-]+)\/([\w-]+)$/.exec(ownerSlashName);
    if (!m) {
        throw new Error(`Expected "${ownerSlashName}" to be in format "owner/name"`);
    }
    return Repository.create(m[1], m[2]);
}
exports.parseBasicRepoReference = parseBasicRepoReference;
function parseRepoReferenceFromURL(url) {
    // https://api.github.com/repos/octocat/Hello-World/something
    const regex = /^https:\/\/[^\/]+\/repos\/([^\/]+)\/([^\/]+)/;
    const match = regex.exec(url);
    if (match == null) {
        throw new Error(`Repository URL was in an unexpected format: ${url}`);
    }
    return Repository.create(match[1], match[2]);
}
exports.parseRepoReferenceFromURL = parseRepoReferenceFromURL;
class User {
    constructor(originalData) {
        this.update(originalData);
    }
    static fromLogin(login) {
        return User.pool.fromKey(login);
    }
    static fromData(data) {
        return User.pool.fromData(data);
    }
    static getCacheKey(login) {
        return `users/${login}`;
    }
    getCacheKey() {
        return User.getCacheKey(this.login);
    }
    update(data) {
        Object.assign(this, {
            login: data.login,
        });
    }
}
User.pool = createPool({
    fetchData: async (login) => {
        return JSON.parse(await client.exec("GET", build_path_1.default("users", login)));
    },
    construct: (data) => new User(data),
    keyFromData: (data) => data.login,
});
exports.User = User;
class Label {
    constructor(repo, data) {
        this.repo = repo;
        this.update([repo, data]);
    }
    static fromData(repo, data) {
        return Label.pool.fromData([repo, data]);
    }
    update(data) {
        Object.assign(this, { name: data[1].name, color: data[1].color });
    }
}
Label.pool = createPool({
    fetchData: async (key) => {
        return JSON.parse(await client.exec("GET", build_path_1.default("repos", key[0].owner, key[0].name, "labels", key[1])));
    },
    construct: (data) => new Label(data[0], data[1]),
    keyFromData: (data) => [data[0], data[1].name],
});
exports.Label = Label;
class IssueComment {
    constructor(originalData, issue) {
        this.originalData = originalData;
        this.issue = issue;
        this.update(originalData);
    }
    static fromData(originalData, issue) {
        return new IssueComment(originalData, issue);
    }
    update(data) {
        Object.assign(this, {
            id: data.id,
            user: User.fromData(data.user),
            created_at: parseDate(data.created_at),
            updated_at: parseDate(data.updated_at),
            body: data.body,
            repository: parseRepoReferenceFromURL(data.issue_url),
        });
    }
    getReactions() {
        // GET /repos/:owner/:repo/issues/comments/:id/reactions
        const repo = parseRepoReferenceFromURL(this.originalData.url);
        return client.fetchIssueCommentReactions(repo, this.id);
    }
}
exports.IssueComment = IssueComment;
class Milestone {
    constructor(originalData) {
        Object.assign(this, {
            id: originalData.id,
            number: originalData.number,
            title: originalData.title,
            description: originalData.description,
            state: originalData.state,
        });
        this.creator = User.fromData(originalData.creator);
        this.created_at = parseDate(originalData.created_at);
        this.updated_at = parseDate(originalData.updated_at);
        this.closed_at = parseDate(originalData.closed_at);
        this.due_on = parseDate(originalData.closed_at);
    }
}
exports.Milestone = Milestone;
class Repository {
    constructor(owner, name) {
        this.owner = owner;
        this.name = name;
    }
    static create(owner, name) {
        const key = owner + "/" + name;
        const result = Repository.cache.get(key);
        if (result === undefined) {
            const repo = new Repository(owner, name);
            Repository.cache.set(key, repo);
            return repo;
        }
        return result;
    }
    get reference() {
        return { name: this.name, owner: this.owner };
    }
}
Repository.cache = new weak_string_map_1.default();
exports.Repository = Repository;
class IssueOrPullRequest {
    static fromData(data) {
        if (data.pull_request) {
            return PullRequest.fromReference(parseRepoReferenceFromURL(data.url), data.number);
        }
        else {
            return Promise.resolve(Issue.fromIssueData(data));
        }
    }
    static getCacheKey(repo, number, isPR) {
        return IssueOrPullRequest.getCacheKeyBasePath(repo, number, isPR) + ".json";
    }
    static getCommentsCacheKey(repo, number, isPR) {
        return IssueOrPullRequest.getCacheKeyBasePath(repo, number, isPR) + ".comments.json";
    }
    static getCacheKeyBasePath(repo, number, isPR) {
        const kind = isPR ? "pull_requests" : "issues";
        // 0000, 1000, 2000, etc
        const thousands = `${Math.floor(number / 1000)}000`;
        // e.g. Microsoft/TypeScript/issues/3000/3123
        return `${repo.owner}/${repo.name}/${kind}/${thousands}/${number}`;
    }
    constructor(originalData) {
        // Copy some fields
        Object.assign(this, {
            number: originalData.number,
            id: originalData.id,
            title: originalData.title,
            body: originalData.body,
            state: originalData.state,
            locked: originalData.locked,
            html_url: originalData.html_url,
        });
        this.repository = parseRepoReferenceFromURL(originalData.url);
        // Parse some dates
        this.created_at = parseDate(originalData.created_at);
        this.updated_at = parseDate(originalData.updated_at);
        this.closed_at = parseDate(originalData.closed_at);
        // Intern some instances
        this.user = User.fromData(originalData.user);
        this.labels = originalData.labels ? originalData.labels.map((l) => Label.fromData(this.repository, l)) : [];
        this.assignees = originalData.assignees.map((user) => User.fromData(user));
    }
    /** Returns a string like 'Microsoft/TypeScript#14' */
    get fullName() {
        return `${this.repository.owner}/${this.repository.name}#${this.number}`;
    }
    update() {
        // TODO: impl
    }
    async getComments() {
        const data = await client.fetchIssueComments(this);
        return data.map((raw) => IssueComment.fromData(raw, this));
    }
    /**
     * Adds a label to an issue or PR.
     */
    addLabel(...labels) {
        return actionRunner_1.addAction(new action_1.Labels.Add(this, labels));
    }
    /**
     * Adds labels to an issue or PR.
     */
    addLabels(labels) {
        return actionRunner_1.addAction(new action_1.Labels.Add(this, labels));
    }
    /**
     * Deletes a label from an issue.
     */
    removeLabel(...labels) {
        return actionRunner_1.addAction(new action_1.Labels.Remove(this, labels));
    }
    /**
     * Deletes labels from an issue.
     */
    removeLabels(labels) {
        return actionRunner_1.addAction(new action_1.Labels.Remove(this, labels));
    }
    /**
     * Sets, exactly, which labels are on an issue
     */
    setLabels(...labels) {
        return actionRunner_1.addAction(new action_1.Labels.Set(this, labels));
    }
    /**
     * Shortcut method for setting/clearing labels with an object literal, e.g.
     * issue.setHasLabels({
     *  "Needs Info": false,
     *  "Ready for Triage": true
     * }).
     * You can specify a null value to cause nothing to happen either way
     */
    setHasLabels(labelMap) {
        for (const key of Object.keys(labelMap)) {
            const alreadyHas = this.labels.some((l) => l.name === key);
            const value = labelMap[key];
            switch (value) {
                case true:
                    if (!alreadyHas) {
                        this.addLabel(key);
                    }
                    break;
                case false:
                    if (alreadyHas) {
                        this.removeLabel(key);
                    }
                    break;
                case null:
            }
        }
    }
    /**
     * Returns true if this issue has the specified label
     */
    hasLabel(labelName) {
        const name = typeof labelName === "string" ? labelName : labelName.name;
        return this.labels.some((l) => l.name === name);
    }
    /**
     * Adds or updates a comment with the specified slug to the issue
     */
    addComment(slug, body) {
        return actionRunner_1.addAction(new action_1.Comments.Add(this, slug, body));
    }
    lock() {
        return actionRunner_1.addAction(new action_1.Issues.Lock(this));
    }
    unlock() {
        return actionRunner_1.addAction(new action_1.Issues.Unlock(this));
    }
    close() {
        return actionRunner_1.addAction(new action_1.Issues.Close(this));
    }
    reopen() {
        return actionRunner_1.addAction(new action_1.Issues.Reopen(this));
    }
}
exports.IssueOrPullRequest = IssueOrPullRequest;
class Issue extends IssueOrPullRequest {
    constructor(data) {
        super(data);
        this.isPullRequest = false;
    }
    static fromData(_data) {
        throw new Error("Don't call me");
    }
    static fromIssueData(data) {
        return new Issue(data);
    }
}
exports.Issue = Issue;
class PullRequest extends IssueOrPullRequest {
    constructor(prData, issueData) {
        super(issueData);
        this.isPullRequest = true;
        this.merge_commit_sha = prData.merge_commit_sha;
        this.merged = prData.merged;
        this.mergeable = prData.mergeable;
        this.mergeable_state = prData.mergeable_state;
        this.comments = prData.comments;
        this.commits = prData.commits;
        this.additions = prData.additions;
        this.deletions = prData.deletions;
        this.changed_files = prData.changed_files;
        this.head = prData.head;
    }
    static async fromReference(repo, number) {
        return new PullRequest(await client.fetchPR(repo, number), await client.fetchIssue(repo, number));
    }
    static fromData(_data) {
        throw new Error("Don't call me");
    }
    static getPRCacheKey(repo, number, isPR) {
        return IssueOrPullRequest.getCacheKeyBasePath(repo, number, isPR) + "pr.json";
    }
    // TODO interning
    static async fromIssueAndPRData(prData, issueData) {
        const issueKey = this.getCacheKey(parseRepoReferenceFromURL(prData.url), prData.number, true);
        const prKey = this.getPRCacheKey(parseRepoReferenceFromURL(prData.url), prData.number, true);
        if (cache) {
            const prCached = await cache.load(prKey);
            const issueCached = await cache.load(issueKey);
            if (prCached.exists && issueCached.exists) {
                const cachedPR = prCached.content;
                const cachedIssue = issueCached.content;
                if (cachedPR.updated_at === prData.updated_at) {
                    return PullRequest.fromPullRequestData(cachedPR, cachedIssue);
                }
            }
        }
        return PullRequest.fromPullRequestData(prData, issueData);
    }
    // TODO interning
    static fromPullRequestData(prData, issueData) {
        return new PullRequest(prData, issueData);
    }
    async getStatusSummary() {
        // GET /repos/:owner/:repo/commits/:ref/status
        return (await client.fetchRefStatusSummary(this.repository.reference, this.head.sha)).state;
    }
    getStatus() {
        // GET /repos/:owner/:repo/commits/:ref/status
        return client.fetchRefStatusSummary(this.repository.reference, this.head.sha);
    }
    getReviews() {
        return client.fetchPRReviews(this.repository.reference, this.number);
    }
    getCommitsRaw() {
        return client.fetchPRCommits(this);
    }
    getFilesRaw() {
        return client.fetchPRFiles(this);
    }
    async getMergeableState() {
        if (this.merged) {
            return null;
        }
        let retryCounter = 5;
        while (this.mergeable === null && retryCounter > 0) {
            const newData = await client.fetchPR(this.repository.reference, this.number);
            if (newData.mergeable === null) {
                console.log(`Sleep 3 seconds and try to get real mergeable state of ${this.number}`);
                await sleep(3000);
                retryCounter--;
            }
            else {
                this.mergeable = newData.mergeable;
                break;
            }
        }
        return this.mergeable;
    }
    merge() {
        return actionRunner_1.addAction(new action_1.PullRequests.Merge(this));
    }
}
exports.PullRequest = PullRequest;
class Project {
    constructor(projectId, columns) {
        this.projectId = projectId;
        this.columns = columns;
    }
    static async create(projectId) {
        const columnsRaw = await client.fetchProjectColumns(projectId);
        const columns = [];
        for (const raw of columnsRaw) {
            columns.push(await ProjectColumn.create(raw.id, raw.name));
        }
        return new Project(projectId, columns);
    }
    setIssueColumn(issue, targetColumn) {
        return actionRunner_1.addAction(new action_1.Issues.SetColumn(issue, this, targetColumn));
    }
    async doSetIssueColumn(issue, targetColumn) {
        for (const sourceColumn of this.columns) {
            const card = sourceColumn.findProjectCardForIssue(issue);
            if (card !== undefined) {
                if (targetColumn === undefined) {
                    await client.deleteProjectCard(card);
                }
                else {
                    if (targetColumn.columnId !== sourceColumn.columnId) {
                        await client.moveProjectCard(card, targetColumn);
                    }
                    return;
                }
            }
        }
        if (targetColumn !== undefined) {
            await client.createProjectCard(targetColumn.columnId, issue);
        }
    }
    findColumnByName(name) {
        for (const c of this.columns) {
            if (typeof name === "string") {
                if (c.name === name) {
                    return c;
                }
            }
            else if (name.test(c.name)) {
                return c;
            }
        }
        return undefined;
    }
}
exports.Project = Project;
class ProjectColumn {
    constructor(columnId, name, cards) {
        this.columnId = columnId;
        this.name = name;
        this.cards = cards.map((c) => new ProjectCard(c));
        console.log(`Initialized column ${name} with ${cards.length} cards`);
    }
    static async create(columnId, name) {
        const cards = await client.fetchProjectColumnCards(columnId);
        return new ProjectColumn(columnId, name, cards);
    }
    findProjectCardForIssue(issue) {
        for (const card of this.cards) {
            if (issue.number === card.getIssueNumber()) {
                return card;
            }
        }
        return undefined;
    }
}
exports.ProjectColumn = ProjectColumn;
class ProjectCard {
    constructor(data) {
        this.data = data;
        this.id = data.id;
    }
    getIssueNumber() {
        // https://api.github.com/repos/DefinitelyTyped/DefinitelyTyped/issues/17902
        const match = /https:\/\/api.github.com\/repos\/\S+\/\S+\/issues\/(\d+)/.exec(this.data.content_url);
        if (match === null) {
            return undefined;
        }
        else {
            return +(match[1]);
        }
    }
}
exports.ProjectCard = ProjectCard;
function createPool(settings) {
    const pool = new weak_string_map_1.default();
    const keyToString = settings.keyToString || ((k) => k);
    const keyToCacheKey = settings.keyToCacheKey || ((k) => k);
    return {
        async fromKey(key) {
            const keyString = keyToString(key);
            const extant = pool.get(keyString);
            if (extant)
                return extant;
            const cacheKey = keyToCacheKey(key);
            const cached = cache && await cache.load(cacheKey);
            if (cached && cached.exists) {
                return settings.construct(cached.content);
            }
            const now = new Date();
            const data = await settings.fetchData(key);
            if (cache)
                cache.save(data, cacheKey, now);
            return settings.construct(data);
        },
        fromData(data) {
            const keyString = keyToString(settings.keyFromData(data));
            const extant = pool.get(keyString);
            if (extant) {
                extant.update(data);
                return extant;
            }
            const result = settings.construct(data);
            pool.set(keyString, result);
            return result;
        },
    };
}
//# sourceMappingURL=github.js.map